@@ -10,6 +10,7 @@ from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
+from async_generator import async_generator, yield_
 from multidict import CIMultiDict, CIMultiDictProxy, istr
 from yarl import URL
 
@@ -40,8 +41,9 @@ def buf():
 
 
 @pytest.fixture
-def protocol(loop):
+def protocol(loop, transport):
     protocol = mock.Mock()
+    protocol.transport = transport
     protocol._drain_helper.return_value = loop.create_future()
     protocol._drain_helper.return_value.set_result(None)
     return protocol
@@ -104,6 +106,14 @@ def test_request_info(make_request):
                                                    req.headers)
 
 
+def test_request_info_with_fragment(make_request):
+    req = make_request('get', 'http://python.org/#urlfragment')
+    assert req.request_info == aiohttp.RequestInfo(
+        URL('http://python.org/'),
+        'GET', req.headers,
+        URL('http://python.org/#urlfragment'))
+
+
 def test_version_err(make_request):
     with pytest.raises(ValueError):
         make_request('get', 'http://python.org/', version='1.c')
@@ -834,10 +844,31 @@ async def test_expect_100_continue_header(loop, conn):
 
 
 async def test_data_stream(loop, buf, conn):
-    @aiohttp.streamer
-    async def gen(writer):
-        await writer.write(b'binary data')
-        await writer.write(b' result')
+    @async_generator
+    async def gen():
+        await yield_(b'binary data')
+        await yield_(b' result')
+
+    req = ClientRequest(
+        'POST', URL('http://python.org/'), data=gen(), loop=loop)
+    assert req.chunked
+    assert req.headers['TRANSFER-ENCODING'] == 'chunked'
+
+    resp = await req.send(conn)
+    assert asyncio.isfuture(req._writer)
+    await resp.wait_for_close()
+    assert req._writer is None
+    assert buf.split(b'\r\n\r\n', 1)[1] == \
+        b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
+    await req.close()
+
+
+async def test_data_stream_deprecated(loop, buf, conn):
+    with pytest.warns(DeprecationWarning):
+        @aiohttp.streamer
+        async def gen(writer):
+            await writer.write(b'binary data')
+            await writer.write(b' result')
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
@@ -874,9 +905,9 @@ async def test_data_file(loop, buf, conn):
 async def test_data_stream_exc(loop, conn):
     fut = loop.create_future()
 
-    @aiohttp.streamer
-    async def gen(writer):
-        await writer.write(b'binary data')
+    @async_generator
+    async def gen():
+        await yield_(b'binary data')
         await fut
 
     req = ClientRequest(
@@ -897,11 +928,38 @@ async def test_data_stream_exc(loop, conn):
     await req.close()
 
 
+async def test_data_stream_exc_deprecated(loop, conn):
+    fut = loop.create_future()
+
+    with pytest.warns(DeprecationWarning):
+        @aiohttp.streamer
+        async def gen(writer):
+            await writer.write(b'binary data')
+            await fut
+
+    req = ClientRequest(
+        'POST', URL('http://python.org/'), data=gen(), loop=loop)
+    assert req.chunked
+    assert req.headers['TRANSFER-ENCODING'] == 'chunked'
+
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
+        fut.set_exception(ValueError)
+
+    loop.create_task(throw_exc())
+
+    await req.send(conn)
+    await req._writer
+    # assert conn.close.called
+    assert conn.protocol.set_exception.called
+    await req.close()
+
+
 async def test_data_stream_exc_chain(loop, conn):
     fut = loop.create_future()
 
-    @aiohttp.streamer
-    async def gen(writer):
+    @async_generator
+    async def gen():
         await fut
 
     req = ClientRequest('POST', URL('http://python.org/'),
@@ -926,12 +984,68 @@ async def test_data_stream_exc_chain(loop, conn):
     await req.close()
 
 
+async def test_data_stream_exc_chain_deprecated(loop, conn):
+    fut = loop.create_future()
+
+    with pytest.warns(DeprecationWarning):
+        @aiohttp.streamer
+        async def gen(writer):
+            await fut
+
+    req = ClientRequest('POST', URL('http://python.org/'),
+                        data=gen(), loop=loop)
+
+    inner_exc = ValueError()
+
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
+        fut.set_exception(inner_exc)
+
+    loop.create_task(throw_exc())
+
+    await req.send(conn)
+    await req._writer
+    # assert connection.close.called
+    assert conn.protocol.set_exception.called
+    outer_exc = conn.protocol.set_exception.call_args[0][0]
+    assert isinstance(outer_exc, ValueError)
+    assert inner_exc is outer_exc
+    assert inner_exc is outer_exc
+    await req.close()
+
+
 async def test_data_stream_continue(loop, buf, conn):
-    @aiohttp.streamer
-    async def gen(writer):
-        await writer.write(b'binary data')
-        await writer.write(b' result')
-        await writer.write_eof()
+    @async_generator
+    async def gen():
+        await yield_(b'binary data')
+        await yield_(b' result')
+
+    req = ClientRequest(
+        'POST', URL('http://python.org/'), data=gen(),
+        expect100=True, loop=loop)
+    assert req.chunked
+
+    async def coro():
+        await asyncio.sleep(0.0001, loop=loop)
+        req._continue.set_result(1)
+
+    loop.create_task(coro())
+
+    resp = await req.send(conn)
+    await req._writer
+    assert buf.split(b'\r\n\r\n', 1)[1] == \
+        b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
+    await req.close()
+    resp.close()
+
+
+async def test_data_stream_continue_deprecated(loop, buf, conn):
+    with pytest.warns(DeprecationWarning):
+        @aiohttp.streamer
+        async def gen(writer):
+            await writer.write(b'binary data')
+            await writer.write(b' result')
+            await writer.write_eof()
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(),
@@ -972,10 +1086,26 @@ async def test_data_continue(loop, buf, conn):
 
 
 async def test_close(loop, buf, conn):
-    @aiohttp.streamer
-    async def gen(writer):
-        await asyncio.sleep(0.00001, loop=loop)
-        await writer.write(b'result')
+    @async_generator
+    async def gen():
+        await asyncio.sleep(0.00001)
+        await yield_(b'result')
+
+    req = ClientRequest(
+        'POST', URL('http://python.org/'), data=gen(), loop=loop)
+    resp = await req.send(conn)
+    await req.close()
+    assert buf.split(b'\r\n\r\n', 1)[1] == b'6\r\nresult\r\n0\r\n\r\n'
+    await req.close()
+    resp.close()
+
+
+async def test_close_deprecated(loop, buf, conn):
+    with pytest.warns(DeprecationWarning):
+        @aiohttp.streamer
+        async def gen(writer):
+            await asyncio.sleep(0.00001, loop=loop)
+            await writer.write(b'result')
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
